"""gr.Button() component."""
from __future__ import annotations

import json
from collections.abc import Callable
from pathlib import Path
from typing import TYPE_CHECKING, Literal

from gradio.components.base import Component
from gradio.events import Events
from gradio_client.documentation import document
from narwhals import from_native
from narwhals.dataframe import DataFrame
from narwhals.typing import IntoFrame

if TYPE_CHECKING:
    from gradio.components import Timer

@document()
class DataFrameDownload(Component):
    """Download button component for client-side DataFrame download as CSV.

    A DataFrame is passed as `value` to this component, serialized to JSON,
    and sent to the frontend. The frontend JavaScript converts the JSON data
    into a CSV string and triggers a download.
    """
    EVENTS = [Events.click]

    def __init__(
        self,
        label: str = "Download DataFrame",
        value: IntoFrame | Callable | None = None,
        *,
        variant: Literal["primary", "secondary", "stop"] = "secondary",
        visible: bool = True,
        size: Literal["sm", "md", "lg"] = "lg",
        icon: str | Path | None = None,
        scale: int | None = None,
        min_width: int | None = None,
        interactive: bool = True,
        elem_id: str | None = None,
        elem_classes: list[str] | str | None = None,
        render: bool = True,
    ):
        """Initialises `DataFrameDownload` button component.

        Args:
            label: Component label text.

            value: A DataFrame to be serialised for download or a callable.

            variant: Style variant; 'primary', 'secondary' or 'stop'.

            visible: Component visibility flag.

            size: Size of the button; 'sm', 'md' or 'lg'.

            icon: URL or path to the icon file to display within the button.

            scale: Relative size compared to adjacent Components.

            min_width: Minimum pixel width (if sufficient screen space).

            interactive: Component interactivity flag.

            elem_id: Optional string assigned as component ID in the HTML DOM.

            elem_classes: CSS classes assigned to component in HTML DOM.

            render: `Blocks` context render flag.
        """
        super().__init__(
            value=value,
            label=label,
            scale=scale,
            min_width=min_width,
            interactive=interactive,
            visible=visible,
            elem_id=elem_id,
            elem_classes=elem_classes,
            render=render
        )

        self.icon = self.serve_static_file(icon)
        self.variant = variant
        self.size = size

    def postprocess(self, value: IntoFrame | None = None) -> str | None:
        """Convert the value returned by a `Gradio` function to JSON.

        Args:
            value: A DataFrame object.
        
        Returns:
            A DataFrame serialized as JSON in 'split' orientation, or None.
        """
        nwdata = from_native(value, pass_through=True)
        if isinstance(nwdata, DataFrame):
            # non-serializable types converted to str - we export to CSV anyway
            return json.dumps(
                {
                    "columns": nwdata.columns,
                    "data": nwdata.to_numpy().tolist()
                },
                default=str
            )
        return

    def preprocess(self, _: str | None = None) -> None:
        """Convert the frontend value to a python-native data structure.

        The function is not used as `DataFrameDownload` does not pass values
        from the frontend.

        _: Not used.

        Returns:
            None.
        """
        return None
    
    def api_info(self) -> dict[str, str]:
        """Provides information for the API documentation."""
        return {
            "type": "string",
            "description": "A JSON string representing the DataFrame."
        }
    
    def example_inputs(self) -> None:
        return None




"""Custom component demo module.

Functions:
    generate_dataframe: Generate a `pandas.DataFrame` for display & download.
"""
import pathlib

import gradio as gr
import numpy as np
from pandas import DataFrame, date_range

from gradio_dataframe_download import DataFrameDownload


def generate_dataframe(nrows: int) -> tuple[DataFrame, DataFrame]:
    """Generate a `pandas.DataFrame` for display & download.
    
    Args:
        nrows: Number of rows to generate for the DataFrame.
    
    Returns:
        A tuple of duplicate generated DataFrame objects.
    """
    data = {
        "ID": np.arange(nrows),
        "Name": map(lambda x: f"NAME_{x}",  range(nrows)),
        "Value": np.random.randn(nrows),
        "Timestamp": date_range(start="2024-01-01", periods=nrows, freq="D"),
        "Category": np.random.choice(range(10), size=nrows),
    }
    dataframe = DataFrame(data)
    return dataframe, dataframe


with gr.Blocks() as demo:
    markdown = pathlib.Path(__file__).parent / "instructions.md"
    gr.Markdown(markdown.read_text())

    with gr.Row(equal_height=True):
        with gr.Column():
            row_slider = gr.Slider(
                minimum=10,
                maximum=1000,
                value=10,
                step=10,
                label="Row count",
                scale=1
            )
        with gr.Column():
            generate_btn = gr.Button("Generate DataFrame", scale=1)
            download_btn = DataFrameDownload(
                label="Download DataFrame",
                variant="primary",
                size="lg",
                scale=1,
            )

    with gr.Column():
        display = gr.Dataframe(label="DataFrame Preview")

    generate_btn.click(
        fn=generate_dataframe, inputs=row_slider, outputs=[display, download_btn]
    )

if __name__ == "__main__":
    demo.launch()



"""gr.Button() component."""
from __future__ import annotations

import json
from collections.abc import Callable
from pathlib import Path
from typing import TYPE_CHECKING, Literal

from gradio.components.base import Component
from gradio.events import Events
from gradio_client.documentation import document
from narwhals import from_native
from narwhals.dataframe import DataFrame
from narwhals.typing import IntoFrame

if TYPE_CHECKING:
    from gradio.components import Timer
from gradio.events import Dependency

@document()
class DataFrameDownload(Component):
    """Download button component for client-side DataFrame download as CSV.

    A DataFrame is passed as `value` to this component, serialized to JSON,
    and sent to the frontend. The frontend JavaScript converts the JSON data
    into a CSV string and triggers a download.
    """
    EVENTS = [Events.click]

    def __init__(
        self,
        label: str = "Download DataFrame",
        value: IntoFrame | Callable | None = None,
        *,
        variant: Literal["primary", "secondary", "stop"] = "secondary",
        visible: bool = True,
        size: Literal["sm", "md", "lg"] = "lg",
        icon: str | Path | None = None,
        scale: int | None = None,
        min_width: int | None = None,
        interactive: bool = True,
        elem_id: str | None = None,
        elem_classes: list[str] | str | None = None,
        render: bool = True,
    ):
        """Initialises `DataFrameDownload` button component.

        Args:
            label: Component label text.

            value: A DataFrame to be serialised for download or a callable.

            variant: Style variant; 'primary', 'secondary' or 'stop'.

            visible: Component visibility flag.

            size: Size of the button; 'sm', 'md' or 'lg'.

            icon: URL or path to the icon file to display within the button.

            scale: Relative size compared to adjacent Components.

            min_width: Minimum pixel width (if sufficient screen space).

            interactive: Component interactivity flag.

            elem_id: Optional string assigned as component ID in the HTML DOM.

            elem_classes: CSS classes assigned to component in HTML DOM.

            render: `Blocks` context render flag.
        """
        super().__init__(
            value=value,
            label=label,
            scale=scale,
            min_width=min_width,
            interactive=interactive,
            visible=visible,
            elem_id=elem_id,
            elem_classes=elem_classes,
            render=render
        )

        self.icon = self.serve_static_file(icon)
        self.variant = variant
        self.size = size

    def postprocess(self, value: IntoFrame | None = None) -> str | None:
        """Convert the value returned by a `Gradio` function to JSON.

        Args:
            value: A DataFrame object.
        
        Returns:
            A DataFrame serialized as JSON in 'split' orientation, or None.
        """
        nwdata = from_native(value, pass_through=True)
        if isinstance(nwdata, DataFrame):
            # non-serializable types converted to str - we export to CSV anyway
            return json.dumps(
                {
                    "columns": nwdata.columns,
                    "data": nwdata.to_numpy().tolist()
                },
                default=str
            )
        return

    def preprocess(self, _: str | None = None) -> None:
        """Convert the frontend value to a python-native data structure.

        The function is not used as `DataFrameDownload` does not pass values
        from the frontend.

        _: Not used.

        Returns:
            None.
        """
        return None
    
    def api_info(self) -> dict[str, str]:
        """Provides information for the API documentation."""
        return {
            "type": "string",
            "description": "A JSON string representing the DataFrame."
        }
    
    def example_inputs(self) -> None:
        return None
    from typing import Callable, Literal, Sequence, Any, TYPE_CHECKING
    from gradio.blocks import Block
    if TYPE_CHECKING:
        from gradio.components import Timer
        from gradio.components.base import Component

    
    def click(self,
        fn: Callable[..., Any] | None = None,
        inputs: Block | Sequence[Block] | set[Block] | None = None,
        outputs: Block | Sequence[Block] | None = None,
        api_name: str | None | Literal[False] = None,
        scroll_to_output: bool = False,
        show_progress: Literal["full", "minimal", "hidden"] = "full",
        show_progress_on: Component | Sequence[Component] | None = None,
        queue: bool | None = None,
        batch: bool = False,
        max_batch_size: int = 4,
        preprocess: bool = True,
        postprocess: bool = True,
        cancels: dict[str, Any] | list[dict[str, Any]] | None = None,
        every: Timer | float | None = None,
        trigger_mode: Literal["once", "multiple", "always_last"] | None = None,
        js: str | Literal[True] | None = None,
        concurrency_limit: int | None | Literal["default"] = "default",
        concurrency_id: str | None = None,
        show_api: bool = True,
        key: int | str | tuple[int | str, ...] | None = None,
        api_description: str | None | Literal[False] = None,
    
        ) -> Dependency:
        """
        Parameters:
            fn: the function to call when this event is triggered. Often a machine learning model's prediction function. Each parameter of the function corresponds to one input component, and the function should return a single value or a tuple of values, with each element in the tuple corresponding to one output component.
            inputs: list of gradio.components to use as inputs. If the function takes no inputs, this should be an empty list.
            outputs: list of gradio.components to use as outputs. If the function returns no outputs, this should be an empty list.
            api_name: defines how the endpoint appears in the API docs. Can be a string, None, or False. If False, the endpoint will not be exposed in the api docs. If set to None, will use the functions name as the endpoint route. If set to a string, the endpoint will be exposed in the api docs with the given name.
            scroll_to_output: if True, will scroll to output component on completion
            show_progress: how to show the progress animation while event is running: "full" shows a spinner which covers the output component area as well as a runtime display in the upper right corner, "minimal" only shows the runtime display, "hidden" shows no progress animation at all
            show_progress_on: Component or list of components to show the progress animation on. If None, will show the progress animation on all of the output components.
            queue: if True, will place the request on the queue, if the queue has been enabled. If False, will not put this event on the queue, even if the queue has been enabled. If None, will use the queue setting of the gradio app.
            batch: if True, then the function should process a batch of inputs, meaning that it should accept a list of input values for each parameter. The lists should be of equal length (and be up to length `max_batch_size`). The function is then *required* to return a tuple of lists (even if there is only 1 output component), with each list in the tuple corresponding to one output component.
            max_batch_size: maximum number of inputs to batch together if this is called from the queue (only relevant if batch=True)
            preprocess: if False, will not run preprocessing of component data before running 'fn' (e.g. leaving it as a base64 string if this method is called with the `Image` component).
            postprocess: if False, will not run postprocessing of component data before returning 'fn' output to the browser.
            cancels: a list of other events to cancel when this listener is triggered. For example, setting cancels=[click_event] will cancel the click_event, where click_event is the return value of another components .click method. Functions that have not yet run (or generators that are iterating) will be cancelled, but functions that are currently running will be allowed to finish.
            every: continously calls `value` to recalculate it if `value` is a function (has no effect otherwise). Can provide a Timer whose tick resets `value`, or a float that provides the regular interval for the reset Timer.
            trigger_mode: if "once" (default for all events except `.change()`) would not allow any submissions while an event is pending. If set to "multiple", unlimited submissions are allowed while pending, and "always_last" (default for `.change()` and `.key_up()` events) would allow a second submission after the pending event is complete.
            js: optional frontend js method to run before running 'fn'. Input arguments for js method are values of 'inputs' and 'outputs', return should be a list of values for output components.
            concurrency_limit: if set, this is the maximum number of this event that can be running simultaneously. Can be set to None to mean no concurrency_limit (any number of this event can be running simultaneously). Set to "default" to use the default concurrency limit (defined by the `default_concurrency_limit` parameter in `Blocks.queue()`, which itself is 1 by default).
            concurrency_id: if set, this is the id of the concurrency group. Events with the same concurrency_id will be limited by the lowest set concurrency_limit.
            show_api: whether to show this event in the "view API" page of the Gradio app, or in the ".view_api()" method of the Gradio clients. Unlike setting api_name to False, setting show_api to False will still allow downstream apps as well as the Clients to use this event. If fn is None, show_api will automatically be set to False.
            key: A unique key for this event listener to be used in @gr.render(). If set, this value identifies an event as identical across re-renders when the key is identical.
            api_description: Description of the API endpoint. Can be a string, None, or False. If set to a string, the endpoint will be exposed in the API docs with the given description. If None, the function's docstring will be used as the API endpoint description. If False, then no description will be displayed in the API docs.
        
        """
        ...
